@@ -79,11 +79,12 @@
 	  (do ((den (/ 1.0 (cosh (* N (acosh alpha)))))
 	       (freq (/ pi N))
 	       (i 0 (+ i 1))
-	       (phase 0.0 (+ phase freq)))
+	       (phase 0.0))
 	      ((= i N))
 	    (let ((val (* den (cos (* N (acos (* alpha (cos phase))))))))
 	      (set! (rl i) (real-part val))
-	      (set! (im i) (imag-part val))))) ;this is always essentially 0.0
+	      (set! (im i) (imag-part val))) ;this is always essentially 0.0
+	    (set! phase (+ phase freq))))
 	(fft rl im -1)            ;direction could also be 1
 	(float-vector-scale! rl (/ 1.0 (float-vector-peak rl)))
 	(do ((i 0 (+ i 1))
@@ -107,9 +108,10 @@
 	       (freq (/ pi N))
 	       (mult -1 (- mult))
 	       (i 0 (+ i 1))
-	       (phase (* -0.5 pi) (+ phase freq)))
+	       (phase (* -0.5 pi)))
 	      ((= i N))
-	    (set! (vals i) (* mult den (cos (* N (acos (* alpha (cos phase)))))))))
+	    (set! (vals i) (* mult den (cos (* N (acos (* alpha (cos phase)))))))
+	    (set! phase (+ phase freq))))
 	;; now take the DFT
 	(let ((pk 0.0)
 	      (w (make-vector N)))
@@ -1825,7 +1827,7 @@ and replaces it with the spectrum given in coeffs")
 	    (pcoeffs (partials->polynomial coeffs))
 	    (peaks (make-vector pairs))
 	    (peaks2 (make-vector pairs))
-	    (flt (make-filter 2 (float-vector 1 -1) (float-vector 0 -0.9)))
+	    (flt (make-filter 2 #r(1 -1) #r(0 -0.9)))
 	    (old-mx (maxamp))
 	    (len (- (or dur (framples snd chn edpos)) beg)))
 	(let ((summer (make-float-vector len))
@@ -2489,7 +2491,13 @@ is assumed to be outside -1.0 to 1.0."))
 
 
 ;;; -------- parallel FM spectrum calculator
-
+;;;
+;;; outside Snd, jn can be accessed via:
+;;;   (require libgsl.scm)
+;;;   (define jn (*libgsl* 'gsl_sf_bessel_Jn))
+;;; or:
+;;;   (require libm.scm)
+;;;   (define jn (*libm* 'jn))
 					;(fm-parallel-component 200 2000.0 (list 2000.0 200.0) (list 0.5 1.0) () () #t)
 
 (define fm-parallel-component 
