@@ -47,7 +47,7 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
 		     ((<= x end)
 		      (set! nenv (append nenv (list x y)))
 		      (if (= x end) (return-early nenv)))
-		     ((> x end)
+		     (else ;(> x end) 
 		      (return-early
 		       (append nenv (list end (envelope-interp end e))))))))
 	   (append nenv (list end lasty))))))))
@@ -58,37 +58,35 @@ end: (window-envelope 1.0 3.0 '(0.0 0.0 5.0 1.0)) -> '(1.0 0.2 3.0 0.6)"))
 (define map-envelopes 
   (let ((documentation "(map-envelopes func env1 env2) maps func over the breakpoints in env1 and env2 returning a new envelope"))
     (lambda (op e1 e2)
-      (let ((xs ()))
-	(let ((at0 
-	       (lambda (e)
-		 (let* ((diff (car e))
-			(len (length e))
-			(lastx (e (- len 2)))
-			(newe (copy e)))
-		   (do ((i 0 (+ i 2)))
-		       ((>= i len) newe)
-		     (let ((x (/ (- (newe i) diff) lastx)))
-		       (set! xs (cons x xs))
-		       (set! (newe i) x)))))))
-	  (if (null? e1)
-	      (at0 e2)
-	      (if (null? e2)
-		  (at0 e1)
-		  (let ((ee1 (at0 e1))
-			(ee2 (at0 e2))
-			(newe ()))
-		    (set! xs (sort! 
-			      (let rem-dup ((lst xs)
-					    (nlst ()))
-				(cond ((null? lst) nlst)
-				      ((member (car lst) nlst) (rem-dup (cdr lst) nlst))
-				      (else (rem-dup (cdr lst) (cons (car lst) nlst)))))
-			      <))
-		    (do ((len (length xs))
-			 (i 0 (+ i 1)))
-			((= i len) newe)
-		      (let ((x (xs i)))
-			(set! newe (append newe (list x (op (envelope-interp x ee1) (envelope-interp x ee2)))))))))))))))
+      (let* ((xs ())
+	     (at0 (lambda (e)
+		    (let* ((diff (car e))
+			   (len (length e))
+			   (lastx (e (- len 2)))
+			   (newe (copy e)))
+		      (do ((i 0 (+ i 2)))
+			  ((>= i len) newe)
+			(let ((x (/ (- (newe i) diff) lastx)))
+			  (set! xs (cons x xs))
+			  (set! (newe i) x)))))))
+	(if (null? e1)
+	    (at0 e2)
+	    (if (null? e2)
+		(at0 e1)
+		(let ((ee1 (at0 e1))
+		      (ee2 (at0 e2))
+		      (newe ()))
+		  (set! xs (let ((lxs (let rem-dup ((lst xs)
+						    (nlst ()))
+					(cond ((null? lst) nlst)
+					      ((member (car lst) nlst) (rem-dup (cdr lst) nlst))
+					      (else (rem-dup (cdr lst) (cons (car lst) nlst)))))))
+			     (sort! lxs <)))
+		  (do ((len (length xs))
+		       (i 0 (+ i 1)))
+		      ((= i len) newe)
+		    (let ((x (xs i)))
+		      (set! newe (append newe (list x (op (envelope-interp x ee1) (envelope-interp x ee2))))))))))))))
 
 
 ;;; -------- multiply-envelopes, add-envelopes
@@ -432,7 +430,7 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
 	      (x-max (e (- (length e) 2))))
 	  (do ((x-incr (* 1.0 (/ (- x-max x-min) xgrid)))
 	       (new-e ())
-	       (x x-min (+ x x-incr)))
+	       (x x-min))
 	      ((>= x x-max)
 	       (reverse new-e))     
 	    (let ((y (envelope-interp x e)))
@@ -441,7 +439,8 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
 				    (+ mn
 				       (* largest-diff
 					  (expt (/ (- y mn) largest-diff) power))))
-				(cons x new-e))))))))))
+				(cons x new-e)))
+	      (set! x (+ x x-incr)))))))))
 
 ;;; rms-envelope
 
@@ -560,4 +559,4 @@ each segment: (powenv-channel '(0 0 .325  1 1 32.0 2 0 32.0))"))
 		    (set! qy ty)
 		    (set! qtx ttx)
 		    (set! qty tty)))))))))
-    
\ No newline at end of file
+    
