@@ -296,7 +296,7 @@
 		  (snd-print (format #f "odd: ~S is using a reserved layer description" mpgfile)))
 	      (let ((chans (if (= channel-mode 3) 1 2))
 		    (mpeg-layer (case layer ((3) 1) ((2)) (else 3)))
-		    (srate (/ (#(44100 48000 32000 0) srate-index)
+		    (srate (/ (#i(44100 48000 32000 0) srate-index)
 			      (case id ((0) 4) ((2)) (else 1)))))
 		(snd-print (format #f "~S: ~A Hz, ~A, MPEG-~A~%" 
 				   mpgfile srate (if (= chans 1) "mono" "stereo") mpeg-layer))
@@ -574,7 +574,7 @@ otherwise it moves the cursor to the first offending sample"))
 	(call-with-exit
 	 (lambda (quit)
 	   (do ((i 0 (+ i 1)))
-	       ((= i len) #t)
+	       ((= i len)) ; returns #t
 	     (if (not (proc (next-sample reader)))
 		 (begin
 		   (set! (cursor) i)
@@ -901,9 +901,10 @@ section: (float-vector->channel (fft-smoother .1 (cursor) 400) (cursor) 400)"))
 		      (do ((incr (let ((offset1 (- old1 new1)))
 				   (if (= offset1 offset0) 0.0 (/ (- offset1 offset0) samps))))
 			   (i 0 (+ i 1))
-			   (trend offset0 (+ trend incr)))
+			   (trend offset0))
 			  ((= i samps))
-			(set! (rl i) (+ (rl i) trend)))
+			(set! (rl i) (+ (rl i) trend))
+			(set! trend (+ trend incr)))
 		      rl))))))))))
 
 
@@ -1062,7 +1063,7 @@ formants, then calls map-channel: (osc-formants .99 (float-vector 400.0 800.0 12
 (define flecho 
   (let ((documentation "(flecho scaler secs) returns a low-pass filtered echo maker: (map-channel (flecho .5 .9) 0 75000)"))
     (lambda (scaler secs)
-      (let ((flt (make-fir-filter :order 4 :xcoeffs (float-vector .125 .25 .25 .125)))
+      (let ((flt (make-fir-filter :order 4 :xcoeffs #r(.125 .25 .25 .125)))
 	    (del (make-delay  (round (* secs (srate))))))
 	(lambda (inval)
 	  (+ inval 
@@ -1558,7 +1559,8 @@ as env moves to 0.0, low-pass gets more intense; amplitude and low-pass amount m
 	    (samp0 0.0)
 	    (samp1 0.0)
 	    (samp2 0.0)
-	    (len (framples)))
+	    (len (framples))
+	    (local-max 0.0))
 	(call-with-exit
 	 (lambda (return)
 	   (do ((ctr loc (+ ctr 1)))
@@ -1566,11 +1568,11 @@ as env moves to 0.0, low-pass gets more intense; amplitude and low-pass amount m
 	     (set! samp0 samp1)
 	     (set! samp1 samp2)
 	     (set! samp2 (next-sample reader))
-	     (let ((local-max (max .1 (moving-max mmax samp0))))
-	       (if (and (> (abs (- samp0 samp1)) local-max)
-			(> (abs (- samp1 samp2)) local-max)
-			(< (abs (- samp0 samp2)) (/ local-max 2)))
-		   (return (- ctr 1)))))))))))
+	     (set! local-max (max .1 (moving-max mmax samp0)))
+	     (if (and (> (abs (- samp0 samp1)) local-max)
+		      (> (abs (- samp1 samp2)) local-max)
+		      (< (abs (- samp0 samp2)) (/ local-max 2)))
+		 (return (- ctr 1))))))))))
 
 (define remove-clicks
   (let ((documentation "(remove-clicks) tries to find and smooth-over clicks"))
@@ -1592,15 +1594,16 @@ as env moves to 0.0, low-pass gets more intense; amplitude and low-pass amount m
       (let ((samp0 0.0)
 	    (samp1 0.0)
 	    (samp2 0.0)
-	    (mmax (make-moving-max 10)))
+	    (mmax (make-moving-max 10))
+	    (local-max 0.0))
 	(lambda (val)
 	  (set! samp0 samp1)
 	  (set! samp1 samp2)
 	  (set! samp2 val)
-	  (let ((local-max (max .1 (moving-max mmax samp0))))
-	    (and (>= (abs (- samp0 samp1)) local-max)
-		 (>= (abs (- samp1 samp2)) local-max)
-		 (<= (abs (- samp0 samp2)) (/ local-max 2)))))))))
+	  (set! local-max (max .1 (moving-max mmax samp0)))
+	  (and (>= (abs (- samp0 samp1)) local-max)
+	       (>= (abs (- samp1 samp2)) local-max)
+	       (<= (abs (- samp0 samp2)) (/ local-max 2))))))))
 
 
 (define zero+
