@@ -473,7 +473,7 @@
 					 (map-chan-over-target-with-sync
 					  (lambda (input-samps) 
 					    (let ((flt (make-fir-filter :order 4 
-									:xcoeffs (float-vector .125 .25 .25 .125)))
+									:xcoeffs #r(.125 .25 .25 .125)))
 						  (del (make-delay (round (* flecho-delay (srate)))))
 						  (genv (make-env (list 0.0 1.0 input-samps 1.0 (+ input-samps 1) 0.0 (+ input-samps 100) 0.0) 
 								  :length (+ input-samps 100))))
@@ -2468,7 +2468,8 @@ the synthesis amplitude, the FFT size, and the radius value."))
 			   (samp0 0.0000)
 			   (samp1 0.0000)
 			   (samp2 0.0000)
-			   (len (framples)))
+			   (len (framples))
+			   (local-max 0.0))
 		       (call-with-exit
 			(lambda (return)
 			  (do ((ctr loc (+ ctr 1)))
@@ -2476,11 +2477,11 @@ the synthesis amplitude, the FFT size, and the radius value."))
 			    (set! samp0 samp1)
 			    (set! samp1 samp2)
 			    (set! samp2 (next-sample reader))
-			    (let ((local-max (max 0.1 (moving-max mmax samp0))))
-			      (if (and (> (abs (- samp0 samp1)) local-max)
-				       (> (abs (- samp1 samp2)) local-max)
-				       (< (abs (- samp0 samp2)) (/ local-max 2)))
-				  (return (- ctr 1)))))))))
+			    (set! local-max (max 0.1 (moving-max mmax samp0)))
+			    (if (and (> (abs (- samp0 samp1)) local-max)
+				     (> (abs (- samp1 samp2)) local-max)
+				     (< (abs (- samp0 samp2)) (/ local-max 2)))
+				(return (- ctr 1))))))))
 		   (let ((click (find-click loc)))
 		     (if click
 			 (begin
@@ -2521,4 +2522,4 @@ the synthesis amplitude, the FFT size, and the radius value."))
   (add-to-menu effects-menu "Reverse" reverse-sound)
   (add-to-menu effects-menu "Null phase" zero-phase)
   
-  ) ; *gtk*
\ No newline at end of file
+  ) ; *gtk*
