@@ -47,118 +47,113 @@
 ;;     editing func with fallback 
 ;;     resynthesis func with fallback
 
-
-(define* (sine-bank amps phases size)
-  (let ((len (or size (length amps)))
-	(sum 0.0))
-    (do ((i 0 (+ i 1)))
-	((= i len))
-      (set! sum (+ sum (* (amps i) (sin (phases i))))))
-    sum))
-
-
 (define pvocoder 
-  (let ((documentation "(pvocoder pv input) is the phase-vocoder generator associated with make-pvocoder"))
+  (let ((documentation "(pvocoder pv input) is the phase-vocoder generator associated with make-pvocoder")
+	;; pvocoder list accessors
+	(pvoc-output (lambda (pv) (pv 0)))
+	(set-pvoc-output (lambda (pv val) (set! (pv 0) val)))
+	(pvoc-interp (lambda (pv) (pv 1)))
+	(pvoc-filptr (lambda (pv) (pv 2)))
+	(set-pvoc-filptr (lambda (pv val) (set! (pv 2) val)))
+	(pvoc-N (lambda (pv) (pv 3)))
+	(pvoc-window (lambda (pv) (pv 4)))
+	(pvoc-D (lambda (pv) (pv 5)))
+	(pvoc-in-data (lambda (pv) (pv 6)))
+	(set-pvoc-in-data (lambda (pv val) (set! (pv 6) val)))
+	(pvoc-ampinc (lambda (pv) (pv 7)))
+	(pvoc-freqs (lambda (pv) (pv 8)))
+	(pvoc-amps (lambda (pv) (pv 9)))
+	(pvoc-phaseinc (lambda (pv) (pv 10)))
+	(pvoc-phases (lambda (pv) (pv 11)))
+	(pvoc-lastphase (lambda (pv) (pv 12)))
+	(pvoc-analyze (lambda (pv) (pv 13)))
+	(pvoc-edit (lambda (pv) (pv 14)))
+	(pvoc-synthesize (lambda (pv) (pv 15)))
+	(sine-bank (lambda* (amps phases size)
+		     (let ((len (or size (length amps)))
+			   (sum 0.0))
+		       (do ((i 0 (+ i 1)))
+			   ((= i len))
+			 (set! sum (+ sum (* (amps i) (sin (phases i))))))
+		       sum)))
+	(pi2 (* 2.0 pi)))
+
     (lambda (pv input)
-      ;; pvocoder list accessors
-      (define (pvoc-output pv) (pv 0))
-      (define (set-pvoc-output pv val) (set! (pv 0) val))
-      (define (pvoc-interp pv) (pv 1))
-      ;(define (set-pvoc-interp pv val) (set! (pv 1) val))
-      (define (pvoc-filptr pv) (pv 2))
-      (define (set-pvoc-filptr pv val) (set! (pv 2) val))
-      (define (pvoc-N pv) (pv 3))
-      (define (pvoc-window pv) (pv 4))
-      (define (pvoc-D pv) (pv 5))
-      (define (pvoc-in-data pv) (pv 6))
-      (define (set-pvoc-in-data pv val) (set! (pv 6) val))
-      (define (pvoc-ampinc pv) (pv 7))
-      (define (pvoc-freqs pv) (pv 8))
-      (define (pvoc-amps pv) (pv 9))
-      (define (pvoc-phaseinc pv) (pv 10))
-      (define (pvoc-phases pv) (pv 11))
-      (define (pvoc-lastphase pv) (pv 12))
-      (define (pvoc-analyze pv) (pv 13))
-      (define (pvoc-edit pv) (pv 14))
-      (define (pvoc-synthesize pv) (pv 15))
-      
-      (let ((pi2 (* 2.0 pi)))
-	
-	(when (>= (pvoc-output pv) (pvoc-interp pv))
-	  ;; get next block of amp/phase info
-	  (let ((N (pvoc-N pv))
-		(D (pvoc-D pv))
-		(amps (pvoc-ampinc pv))
-		(freqs (pvoc-freqs pv))
-		(filptr (pvoc-filptr pv)))
-	    
-	    (if (pvoc-analyze pv)
-		((pvoc-analyze pv) pv input)
-		;; if no analysis func:
-		(begin
-		  (fill! freqs 0.0)
-		  (set-pvoc-output pv 0)
-		  (if (not (pvoc-in-data pv))
+      (when (>= (pvoc-output pv) (pvoc-interp pv))
+	;; get next block of amp/phase info
+	(let ((N (pvoc-N pv))
+	      (D (pvoc-D pv))
+	      (amps (pvoc-ampinc pv))
+	      (freqs (pvoc-freqs pv))
+	      (filptr (pvoc-filptr pv)))
+	  
+	  (if (pvoc-analyze pv)
+	      ((pvoc-analyze pv) pv input)
+	      ;; if no analysis func:
+	      (begin
+		(fill! freqs 0.0)
+		(set-pvoc-output pv 0)
+		(if (not (pvoc-in-data pv))
+		    (begin
+		      (set-pvoc-in-data pv (make-float-vector N))
+		      (do ((i 0 (+ i 1)))
+			  ((= i N))
+			(set! ((pvoc-in-data pv) i) (input))))
+		    (let ((indat (pvoc-in-data pv)))
+		      ;; extra loop here since I find the optimized case confusing (we could dispense with the data move)
+		      (float-vector-move! indat 0 D)
+		      (do ((i (- N D) (+ i 1)))
+			  ((= i N))
+			(set! (indat i) (input)))))
+		(let ((buf (modulo filptr N)))
+		  (if (= buf 0)
 		      (begin
-			(set-pvoc-in-data pv (make-float-vector N))
-			(do ((i 0 (+ i 1)))
-			    ((= i N))
-			  (set! ((pvoc-in-data pv) i) (input))))
-		      (let ((indat (pvoc-in-data pv)))
-			;; extra loop here since I find the optimized case confusing (we could dispense with the data move)
-			(float-vector-move! indat 0 D)
-			(do ((i (- N D) (+ i 1)))
-			    ((= i N))
-			  (set! (indat i) (input)))))
-		  (let ((buf (modulo filptr N)))
-		    (if (= buf 0)
-			(begin
-			  (fill! amps 0.0)
-			  (float-vector-add! amps (pvoc-in-data pv))
-			  (float-vector-multiply! amps (pvoc-window pv)))
-			(do ((k 0 (+ k 1)))
-			    ((= k N))
-			  (set! (amps buf) (* ((pvoc-window pv) k) ((pvoc-in-data pv) k)))
-			  (set! buf (+ 1 buf))
-			  (if (= buf N) (set! buf 0)))))
-		  (set-pvoc-filptr pv (+ filptr D))
-		  (mus-fft amps freqs N 1)
-		  (rectangular->polar amps freqs)))
-	    
-	    (if (pvoc-edit pv)
-		((pvoc-edit pv) pv)
-		(let ((lp (pvoc-lastphase pv))
-		      (pscl (/ 1.0 D))
-		      (kscl (/ pi2 N))
-		      (lim (floor (/ N 2))))
-		  ;; if no editing func:
-		  (do ((k 0 (+ k 1)))
-		      ((= k lim))
-		    (let ((phasediff (remainder (- (freqs k) (lp k)) pi2)))
-		      (float-vector-set! lp k (freqs k))
-		      (if (> phasediff pi) (set! phasediff (- phasediff pi2))
-			  (if (< phasediff (- pi)) (set! phasediff (+ phasediff pi2))))
-		      (set! (freqs k) (+ (* pscl phasediff) (* k kscl)))))))
-	    
-	    (let ((scl (/ 1.0 (pvoc-interp pv))))
-	      (float-vector-subtract! amps (pvoc-amps pv))
-	      (float-vector-subtract! freqs (pvoc-phaseinc pv))
-	      (float-vector-scale! amps scl)
-	      (float-vector-scale! freqs scl)
-	      )))
-	
-	(set-pvoc-output pv (+ 1 (pvoc-output pv)))
-	
-	(if (pvoc-synthesize pv)
-	    ((pvoc-synthesize pv) pv)
-	    ;; if no synthesis func:
-	    ;; synthesize next sample
-	    (begin
-	      (float-vector-add! (pvoc-amps pv) (pvoc-ampinc pv))
-	      (float-vector-add! (pvoc-phaseinc pv) (pvoc-freqs pv))
-	      (float-vector-add! (pvoc-phases pv) (pvoc-phaseinc pv))
-	      (sine-bank (pvoc-amps pv) (pvoc-phases pv))))
-	))))
+			(fill! amps 0.0)
+			(float-vector-add! amps (pvoc-in-data pv))
+			(float-vector-multiply! amps (pvoc-window pv)))
+		      (do ((k 0 (+ k 1)))
+			  ((= k N))
+			(set! (amps buf) (* ((pvoc-window pv) k) ((pvoc-in-data pv) k)))
+			(set! buf (+ 1 buf))
+			(if (= buf N) (set! buf 0)))))
+		(set-pvoc-filptr pv (+ filptr D))
+		(mus-fft amps freqs N 1)
+		(rectangular->polar amps freqs)))
+	  
+	  (if (pvoc-edit pv)
+	      ((pvoc-edit pv) pv)
+	      (let ((lp (pvoc-lastphase pv))
+		    (pscl (/ 1.0 D))
+		    (kscl (/ pi2 N))
+		    (lim (floor (/ N 2))))
+		;; if no editing func:
+		(do ((k 0 (+ k 1)))
+		    ((= k lim))
+		  (let ((phasediff (remainder (- (freqs k) (lp k)) pi2)))
+		    (float-vector-set! lp k (freqs k))
+		    (if (> phasediff pi) (set! phasediff (- phasediff pi2))
+			(if (< phasediff (- pi)) (set! phasediff (+ phasediff pi2))))
+		    (set! (freqs k) (+ (* pscl phasediff) (* k kscl)))))))
+	  
+	  (let ((scl (/ 1.0 (pvoc-interp pv))))
+	    (float-vector-subtract! amps (pvoc-amps pv))
+	    (float-vector-subtract! freqs (pvoc-phaseinc pv))
+	    (float-vector-scale! amps scl)
+	    (float-vector-scale! freqs scl)
+	    )))
+      
+      (set-pvoc-output pv (+ 1 (pvoc-output pv)))
+      
+      (if (pvoc-synthesize pv)
+	  ((pvoc-synthesize pv) pv)
+	  ;; if no synthesis func:
+	  ;; synthesize next sample
+	  (begin
+	    (float-vector-add! (pvoc-amps pv) (pvoc-ampinc pv))
+	    (float-vector-add! (pvoc-phaseinc pv) (pvoc-freqs pv))
+	    (float-vector-add! (pvoc-phases pv) (pvoc-phaseinc pv))
+	    (sine-bank (pvoc-amps pv) (pvoc-phases pv))))
+      )))
 
 #|
 (let* ((ind (open-sound "oboe.snd"))
@@ -235,22 +230,22 @@
   algorithm to the current sound (i.e. fft analysis, oscil bank resynthesis). 'pitch'
   specifies the pitch transposition ratio, 'time' - specifies the time dilation ratio,
   'gate' specifies a resynthesis gate in dB (partials with amplitudes lower than
-  the gate value will not be synthesized), 'hoffset is a pitch offset in Hz."))
+  the gate value will not be synthesized), 'hoffset is a pitch offset in Hz.")
+	(pi2 (* 2.0 pi)))
     
     (lambda* ((fftsize 512) (overlap 4) (time 1.0)
 	      (pitch 1.0) (gate 0.0) (hoffset 0.0)
 	      (snd 0) (chn 0))
-      (let* ((len (framples))
-	     (N2 (floor (/ fftsize 2)))
-	     (window (make-fft-window hamming-window fftsize))
-	     (lastamp (make-float-vector N2))
-	     (lastfreq (make-float-vector N2))
-	     (in-data (channel->float-vector 0 (* fftsize 2) snd chn))
-	     (obank (make-oscil-bank lastfreq (make-float-vector N2) lastamp)))
-	(let ((pi2 (* 2 pi))
+      (let ((len (framples))
+	    (N2 (floor (/ fftsize 2)))
+	    (window (make-fft-window hamming-window fftsize))
+	    (in-data (channel->float-vector 0 (* fftsize 2) snd chn)))
+	(let ((lastamp (make-float-vector N2))
+	      (lastfreq (make-float-vector N2))
 	      (outlen (floor (* time len)))
 	      (interp (* (floor (/ fftsize overlap)) time)))
-	  (let ((filptr 0)
+	  (let ((obank (make-oscil-bank lastfreq (make-float-vector N2) lastamp))
+		(filptr 0)
 		(D (floor (/ fftsize overlap)))
 		(syngate (if (= 0.0 gate)    ; take a resynthesis gate specificed in dB, convert to linear amplitude
 			     0.0000
@@ -331,4 +326,4 @@
 	      (float-vector-add! lastfreq freqinc)
 	      (float-vector-set! out-data i (oscil-bank obank))
 	      (set! output (+ 1 output)))
-	    (float-vector->channel out-data 0 (max len outlen))))))))
\ No newline at end of file
+	    (float-vector->channel out-data 0 (max len outlen))))))))
