@@ -7,17 +7,17 @@
 ;;; ---------------- gl-info ----------------
 (define gl-info
   ;; taken loosely from glxvisuals.c
-  (let ((documentation "(gl-info) prints out GL-related info"))
+  (let ((documentation "(gl-info) prints out GL-related info")
+	(class-of (lambda (n)
+		    (and (number? n)
+			 (copy (cond ((= n (*motif* 'StaticGray)) "static-gray")
+				     ((= n (*motif* 'GrayScale)) "gray-scale")
+				     ((= n (*motif* 'StaticColor)) "static-color")
+				     ((= n (*motif* 'PseudoColor)) "pseudo-color")
+				     ((= n (*motif* 'TrueColor)) "true-color")
+				     ((= n (*motif* 'DirectColor)) "direct-color")
+				     (#t "??")))))))
     (lambda ()
-      (define (class-of n)
-	(and (number? n)
-	     (cond ((= n (*motif* 'StaticGray)) "static-gray")
-		   ((= n (*motif* 'GrayScale)) "gray-scale")
-		   ((= n (*motif* 'StaticColor)) "static-color")
-		   ((= n (*motif* 'PseudoColor)) "pseudo-color")
-		   ((= n (*motif* 'TrueColor)) "true-color")
-		   ((= n (*motif* 'DirectColor)) "direct-color")
-		   (#t "??"))))
       (let* ((cx (snd-gl-context))
 	     (dpy ((*motif* 'XtDisplay) (cadr (main-widgets))))
 	     (version (glXQueryVersion dpy 0 0)))
@@ -326,9 +326,9 @@
 	    (let ((rl (channel->float-vector (left-sample) 512))
 		  (im (make-float-vector 512)))
 	      (mus-fft rl im)
-	      (let ((peak (* 2 (max (float-vector-peak rl) (float-vector-peak im)))))
-		(float-vector-scale! rl (/ 1.0 peak))
-		(float-vector-scale! im (/ 1.0 peak)))
+	      (let ((peak (/ (* 2 (max (float-vector-peak rl) (float-vector-peak im))))))
+		(float-vector-scale! rl peak)
+		(float-vector-scale! im peak))
 	      ;; display each element in the complex plane rotated to stack along the x axis
 	      (glNewList gl-list GL_COMPILE)
 	      (glBegin GL_LINES)
