@@ -154,7 +154,7 @@ static bool prefs_dialog_error_is_posted = false;
 static void post_prefs_dialog_error(const char *message, void *data)
 {
   gtk_window_set_title(GTK_WINDOW(preferences_dialog), (char *)message);
-  prefs_dialog_error_is_posted = (message != NULL);
+  prefs_dialog_error_is_posted = (bool)message;
 }
 
 
@@ -1092,11 +1092,23 @@ static void pixel_to_rgb(color_t pix, float *r, float *g, float *b)
 static void display_color(GtkWidget *w, color_t pixel)
 {
   cairo_t *cr;
+#if GTK_CHECK_VERSION(3, 22, 0)
+  GdkWindow *window;
+  GdkDrawingContext *context;
+  window = WIDGET_TO_WINDOW(w);
+  context = gdk_window_begin_draw_frame(window, gdk_window_get_visible_region(window));
+  cr = gdk_drawing_context_get_cairo_context(context);
+#else
   cr = gdk_cairo_create(WIDGET_TO_WINDOW(w));
+#endif
   cairo_set_source_rgba(cr, pixel->red, pixel->green, pixel->blue, pixel->alpha);
   cairo_rectangle(cr, 0, 0, widget_width(w), widget_height(w));
   cairo_fill(cr);
+#if GTK_CHECK_VERSION(3, 22, 0)
+  gdk_window_end_draw_frame(window, context);
+#else
   cairo_destroy(cr);
+#endif
 }
 #endif
 
@@ -1630,15 +1642,17 @@ widget_t make_preferences_dialog(void)
 
   preferences_dialog = snd_gtk_dialog_new();
 #if GTK_CHECK_VERSION(3, 14, 0)
-  gtk_window_set_transient_for(GTK_WINDOW(preferences_dialog), GTK_WINDOW(MAIN_SHELL(ss)));
+  gtk_window_set_transient_for(GTK_WINDOW(preferences_dialog), GTK_WINDOW(main_shell(ss)));
 #endif
   gtk_window_set_title(GTK_WINDOW(preferences_dialog), "Preferences");
   sg_make_resizable(preferences_dialog);
   /* gtk_container_set_border_width (GTK_CONTAINER(preferences_dialog), 10); */
   gtk_widget_realize(preferences_dialog);
 
+#if (!GTK_CHECK_VERSION(3, 22, 0))
   if ((STARTUP_WIDTH < gdk_screen_width()) &&
       (STARTUP_HEIGHT < gdk_screen_height()))
+#endif
     gtk_window_resize(GTK_WINDOW(preferences_dialog), STARTUP_WIDTH, STARTUP_HEIGHT);
 
   help_button = gtk_dialog_add_button(GTK_DIALOG(preferences_dialog), "Help", GTK_RESPONSE_NONE);
