@@ -424,6 +424,7 @@
 			      0)))))
       (lambda (hook)
 	;; (show-disk-space snd) adds a label to snd's status-area area showing the current free space (for use with after-open-hook)
+	;; (set! (hook-functions after-open-hook) (list (*motif* 'show-disk-space)))
 	
 	(let* ((snd (hook 'snd))
 	       (previous-label (let find-if ((pred (lambda (n)
@@ -450,7 +451,7 @@
 ;;; (remove-main-menu 5) removes the Help menu
   
   (define remove-main-menu 
-    (let ((documentation "(remove-main-menu menu) removes the specified top-level menu: (remove-main-menu 5) removes the Help menu"))
+    (let ((documentation "(remove-main-menu menu) removes the specified top-level menu: ((*gtk* 'remove-main-menu) 5) removes the Help menu"))
       (lambda (menu)
 	(gtk_widget_hide ((menu-widgets) menu)))))
   
@@ -759,7 +760,6 @@
   (define (make-variables-dialog)
     (set! variables-dialog (gtk_dialog_new))
     (gtk_window_set_title (GTK_WINDOW variables-dialog) "Variables")
-    ;(gtk_container_set_border_width (GTK_CONTAINER variables-dialog) 10)
     (gtk_window_set_default_size (GTK_WINDOW variables-dialog) -1 -1)
     (gtk_window_set_resizable (GTK_WINDOW variables-dialog) #t)
     (gtk_widget_realize variables-dialog)
@@ -832,7 +832,6 @@
   (define variable-display 
     (let ((force-update (lambda (wid)
 			  (gdk_window_invalidate_rect (GDK_WINDOW (gtk_widget_get_window (GTK_WIDGET wid))) (list 'GdkRectangle_ 0) #t)
-			  ;;(gdk_window_process_updates (GDK_WINDOW (gtk_widget_get_window (GTK_WIDGET wid))) #t)
 			  ))
 	  (widget? (lambda (w) 
 		     (and (pair? w) 
@@ -871,4 +870,4 @@
   (define (notebook-with-top-tabs)
     (gtk_notebook_set_tab_pos (GTK_NOTEBOOK ((main-widgets) 5)) GTK_POS_TOP))
 
-  ) ; *gtk*
\ No newline at end of file
+  ) ; *gtk*
