@@ -21,6 +21,8 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdint.h>
+#include <inttypes.h>
 
 #include "_sndlib.h"
 #include "xen.h"
@@ -53,11 +55,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "16-Jan-17"
+#define SND_DATE "3-Aug-17"
 #ifndef SND_VERSION
-#define SND_VERSION "17.1"
+#define SND_VERSION "17.6"
 #endif
 #define SND_MAJOR_VERSION "17"
-#define SND_MINOR_VERSION "1"
+#define SND_MINOR_VERSION "6"
 
 #endif
