@@ -67,8 +67,8 @@
 				(values "~%    ~S" (car x)))))
 		(format p "~%"))
 	     (set! history (cons (car x) history))
-	     (set! lines (cons (pair-line-number (car x)) lines))
-	     (set! files (cons (pair-filename (car x)) files)))))
+	     (set! lines (cons (and (pair? (car x)) (pair-line-number (car x))) lines))
+	     (set! files (cons (and (pair? (car x)) (pair-filename (car x))) files)))))
        
        ;; show the enclosing contexts
        (let ((old-print-length (*s7* 'print-length)))
@@ -734,6 +734,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 		     (append rest (map (lambda (subset) (cons (car set) subset)) rest)))))))))
 
 ;;; ----------------
+;;;   now superseded by built-in type-of
 (define ->predicate
   (let ((predicates (list integer? rational? real? complex? number?
 			  byte-vector? string?
@@ -786,12 +787,12 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
        (cond ,@(map (lambda (clause)         
 		      (if (memq (car clause) '(#t else))
 			  clause
-			  (if (= (length (car clause)) 1)
-			      (cons (list (caar clause) obj) (cdr clause))
-			      (cons (cons 'or (map (lambda (type)
+			  (cons (if (= (length (car clause)) 1)
+				    (list (caar clause) obj)
+				    (cons 'or (map (lambda (type)
 						     (list type obj))
-						   (car clause)))
-				    (cdr clause)))))
+						   (car clause))))
+				(cdr clause))))
 		    clauses)))))
 
 
@@ -2128,6 +2129,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 			      (rootlet))
 		    (sublet lt)))))))
 
+;;; (sandbox '(let ((x 1)) (+ x 2))) -> 3
 
 (define sandbox 
   (let ((documentation "(sandbox code) evaluates code in an environment where nothing outside that code can be affected by its evaluation.")
@@ -2169,7 +2171,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 	      float-vector-ref int-vector make-int-vector int-vector-set! int-vector-ref string->byte-vector 
 	      byte-vector make-byte-vector hash-table hash-table* make-hash-table hash-table-ref 
 	      hash-table-set! hash-table-entries cyclic-sequences call/cc call-with-current-continuation 
-	      call-with-exit apply for-each map dynamic-wind values 
+	      call-with-exit apply for-each map dynamic-wind values type-of
 	      catch throw error procedure-documentation procedure-signature help procedure-source
 	      procedure-setter arity aritable? not eq? eqv? equal? morally-equal? s7-version
 	      dilambda make-hook hook-functions stacktrace tree-leaves tree-memq object->let
@@ -2179,7 +2181,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 	      quote if begin let let* letrec letrec* cond case or and do set! unless when else 
 	      with-let with-baffle
 	      lambda lambda* define define* 
-	      define-macro define-macro* define-bacro define-bacro*))
+	      define-macro define-macro* define-bacro define-bacro* macroexpand)) ; not sure about macroexpand
 	   ht))
 	(baddies (list #_eval #_eval-string #_load #_autoload #_define-constant #_define-expansion #_require
 		       #_string->symbol #_symbol->value #_symbol->dynamic-value #_symbol-table #_symbol #_keyword->symbol 
@@ -2242,8 +2244,9 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 			       ((hook-functions *error-hook*) ())
 			       ((hook-functions *read-error-hook*) ())
 			       ((hook-functions *rootlet-redefinition-hook*) ())
-			       ((current-output-port) *stdout*)
-			       ((current-error-port) *stderr*))
+			       (reader-cond ((not (provided? 'pure-s7))
+					     ((current-output-port) *stdout*)
+					     ((current-error-port) *stderr*))))
 	       (catch #t
 		 (lambda ()
 		   (eval new-code (sublet (rootlet) (unlet))))
@@ -2254,3 +2257,4 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 			       (apply format #f (cadr args)))
 			     (lambda args
 			       (copy "?"))))))))))))
+
