@@ -57,6 +57,7 @@
       ((= i 25))
     (read-line))
   ;; read about 500 lines and store in a hash table as (func . timing)
+  ;; names can match!
   (let ((h (make-hash-table)))
     (call-with-exit
      (lambda (quit)
@@ -75,10 +76,17 @@
 			    (let ((num (string->number-ignoring-commas (substring line k end))))
 			      (when num
 				(let ((func-end (char-position #\space line (+ end 2))))
-				  (when (and (number? func-end)
+				  (when (and (integer? func-end)
 					     (> func-end (+ end 2)))
-				    (let ((func (string->symbol (substring line (+ end 2) func-end))))
-				      (set! (h func) num))))))))))))))))
+				    (let ((func (substring line (+ end 2) func-end)))
+				      (let ((colon-pos (char-position #\: func)))
+					(if (integer? colon-pos)
+					    (let ((isra-pos (char-position #\. func colon-pos)))
+					      (if (integer? isra-pos)
+						  (set! func (substring func 0 isra-pos))))))
+				      (let ((sym (string->symbol func)))
+					(let ((curval (h sym)))
+					  (set! (h sym) (+ (or curval 0) num)))))))))))))))))))
     h))
     
 
@@ -191,8 +199,8 @@
 
 (define (combine-latest)
   (let ((file-names (list "v-eq" "v-iter" "v-map" "v-form" "v-hash" "v-cop"
-			  "v-lg" "v-gen" "v-auto" "v-index" "v-call" "v-all" 
-			  "v-test" "/home/bil/test/bench/src/v-b")))
+			  "v-lt" "v-gen" "v-auto" "v-index" "v-call" "v-all" 
+			  "v-test" "/home/bil/test/scheme/bench/src/v-b")))
 
     (define (next-file f)
       (let ((name (system (format #f "ls -t ~A*" f) #t)))
@@ -209,5 +217,5 @@
 #|
 (combine "v-call53" "v-map52" "v-all98" "v-hash31" "v-gen72" "v-auto51" 
 	 "v-lg73" "v-cop55" "v-form66" "v-eq46" "v-test57" "v-iter70" "v-index22"
-         "/home/bil/test/bench/src/v-b28")
+         "/home/bil/test/scheme/bench/src/v-b28")
 |#
