@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
+#include <inttypes.h>
 
 #include "s7.h"
 
@@ -156,6 +157,22 @@ static s7_pointer plus(s7_scheme *sc, s7_pointer args)
   return(TO_S7_INT(2 * s7_integer(s7_car(args)) + s7_integer(s7_car(s7_cdr(args)))));
 }
 
+static s7_pointer plus1(s7_scheme *sc, s7_pointer args) /* check recursion in "unsafe" case */
+{
+  s7_pointer d;
+  if (s7_integer(s7_car(args)) == 4)
+    d = s7_apply_function_star(sc, s7_name_to_value(sc, "plus1"), 
+			       s7_list(sc, 3, 
+				       s7_make_integer(sc, 1), 
+				       s7_make_integer(sc, 2), 
+				       s7_make_integer(sc, 3)));
+  else d = s7_make_integer(sc, 0);
+  return(s7_make_integer(sc, s7_integer(s7_car(args)) + 
+			 s7_integer(s7_cadr(args)) + 
+			 s7_integer(s7_caddr(args)) + 
+			 s7_integer(d)));
+}
+
 static s7_pointer mac_plus(s7_scheme *sc, s7_pointer args)
 {
   /* (define-macro (plus a b) `(+ ,a ,b)) */
@@ -998,6 +1015,18 @@ int main(int argc, char **argv)
     {fprintf(stderr, "%d: %s is not an integer?\n", __LINE__, s1 = TO_STR(p)); free(s1);}
   if (s7_integer(p) != 4)
     {fprintf(stderr, "%d: %s is not 4?\n", __LINE__, s1 = TO_STR(p)); free(s1);}
+
+  s7_define_function_star(sc, "plus1", plus1, "a b c", "an example of define* from C");
+  {
+    s7_int val;
+    val = s7_integer(s7_apply_function_star(sc, s7_name_to_value(sc, "plus1"),
+					    s7_list(sc, 3,
+						    s7_make_integer(sc, 4),
+						    s7_make_integer(sc, 5),
+						    s7_make_integer(sc, 6))));
+    if (val != 21)
+      fprintf(stderr, "plus1: %" PRId64 "\n", val);
+  }
   
   p = s7_apply_function(sc, s7_name_to_value(sc, "plus"), s7_cons(sc, s7_make_keyword(sc, "blue"), s7_cons(sc, TO_S7_INT(2), s7_nil(sc))));
   if (!s7_is_integer(p))
@@ -1082,11 +1111,11 @@ int main(int argc, char **argv)
     dims = s7_vector_dimensions(p1);
     offs = s7_vector_offsets(p1);
     els = s7_vector_elements(p1);
-    if (dims[0] != 2) fprintf(stderr, "%d: dims[0]: %lld?\n", __LINE__, dims[0]);
-    if (dims[1] != 3) fprintf(stderr, "%d: dims[1]: %lld?\n", __LINE__, dims[1]);
-    if (dims[2] != 4) fprintf(stderr, "%d: dims[2]: %lld?\n", __LINE__, dims[2]);
-    if (offs[0] != 12) fprintf(stderr, "%d: offs[0]: %lld?\n", __LINE__, offs[0]);
-    if (offs[1] != 4) fprintf(stderr, "%d: offs[1]: %lld?\n", __LINE__, offs[1]);
+    if (dims[0] != 2) fprintf(stderr, "%d: dims[0]: %" PRId64 "?\n", __LINE__, dims[0]);
+    if (dims[1] != 3) fprintf(stderr, "%d: dims[1]: %" PRId64 "?\n", __LINE__, dims[1]);
+    if (dims[2] != 4) fprintf(stderr, "%d: dims[2]: %" PRId64 "?\n", __LINE__, dims[2]);
+    if (offs[0] != 12) fprintf(stderr, "%d: offs[0]: %" PRId64 "?\n", __LINE__, offs[0]);
+    if (offs[1] != 4) fprintf(stderr, "%d: offs[1]: %" PRId64 "?\n", __LINE__, offs[1]);
     if (s7_integer(p = els[12 + 4 + 1]) != 32)
       {fprintf(stderr, "%d: %s is not 32?\n", __LINE__, s1 = TO_STR(p)); free(s1);}
   }
@@ -1441,7 +1470,7 @@ int main(int argc, char **argv)
     iter = s7_make_iterator(sc, s7_list(sc, 3, TO_S7_INT(1), TO_S7_INT(2), TO_S7_INT(3)));
     if (!s7_is_iterator(iter))
       fprintf(stderr, "%d: %s is not an interator\n", __LINE__, TO_STR(iter));
-    if (s7_iterator_is_at_end(iter))
+    if (s7_iterator_is_at_end(sc, iter))
       fprintf(stderr, "%d: %s is prematurely done\n", __LINE__, TO_STR(iter));
     x = s7_iterate(sc, iter);
     if ((!s7_is_integer(x)) || (s7_integer(x) != 1))
@@ -1453,7 +1482,7 @@ int main(int argc, char **argv)
     if ((!s7_is_integer(x)) || (s7_integer(x) != 3))
       fprintf(stderr, "%d: %s should be 3\n", __LINE__, TO_STR(x));
     x = s7_iterate(sc, iter);
-    if ((x != s7_eof_object(sc)) || (!s7_iterator_is_at_end(iter)))
+    if ((x != s7_eof_object(sc)) || (!s7_iterator_is_at_end(sc, iter)))
       fprintf(stderr, "%d: %s should be #<eof> and iter should be done\n", __LINE__, TO_STR(x));
   }
 
