@@ -99,14 +99,13 @@
 	       x))
 	 sequence)))
 
-#|
+
 (define (count-table commands)
   (do ((count 0 (+ count 1))
        (c (memq 'table commands) (memq 'table (cdr c))))
       ((not c) count)))
-;;; but sadly the for-each version below is faster.
-|#
 
+#|
 (define (count-table commands)
   (let ((count 0))
     (for-each
@@ -115,7 +114,7 @@
 	   (set! count (+ count 1))))
      commands)
     count))
-
+|#
 
 
 (define (string</* a b)
@@ -477,7 +476,8 @@
        (when (pair? binding)
 	 (let ((symbol (car binding))
 	       (value (cdr binding)))
-	   (when (procedure? value)
+	   (when (and (procedure? value)
+		      (let? (funclet value)))
 	     (let ((file (let ((addr (with-let (funclet value) __func__)))
 			   ;; this misses scheme-side pws because their environment is (probably) the global env
 			   (and (pair? addr)
@@ -549,7 +549,6 @@
     (*clm-player* "ws.scm")
     (*clm-notehook* "ws.scm")
     (*clm-with-sound-depth* "ws.scm")
-    (*clm-default-frequency* "ws.scm")
     (*clm-delete-reverb* "ws.scm")
     (*to-snd* "ws.scm")
     (*clm-search-list* "ws.scm")
@@ -1015,6 +1014,21 @@
 	  ((= i n))
         (set! (tnames ctr)
 	      (clean-and-downcase-first-char (names i) capitalized (files i)))
+#|	
+	;; catch forgotten stuff
+	(let ((old-name (ind-sortby (tnames i))))
+	  (when (not (char-position " ,(" old-name))
+	    (let ((&pos (char-position #\& old-name)))
+	      (when &pos
+		(set! old-name (string-append (substring old-name 0 &pos) (string #\>) (substring old-name (+ &pos 4)))))
+	      (set! old-name (string->symbol old-name))
+	      (unless (or (memq old-name '(list->float-vector float-vector-fill! float-vector-length float-vector-reverse! float-vector->list ;vct.c but not s7
+					   min-db filter-control-in-db enved-in-db ; -dB in s7
+					   init-ladspa list-ladspa analyse-ladspa ladspa-descriptor apply-ladspa
+					   bignum? bignum))
+			  (defined? old-name))
+		  (format *stderr* "??: ~A~%" old-name)))))
+|#
 	(if (positive? (length (ind-sortby (tnames ctr))))
 	    (set! ctr (+ ctr 1))))
 
@@ -1115,7 +1129,7 @@
 	       (ctr 0)
 	       (offset (ceiling (/ n cols)))
 	       (i 0 (+ i 1)))
-	      ((>= row offset))
+	      ((= row offset))
 	    (let ((x (+ row (* ctr offset))))
 	      (if (>= x n)
 		  (format ofil "~%")
@@ -1620,3 +1634,5 @@
 
 (s7-version)
 (exit)
+
+
