@@ -32,14 +32,14 @@
 ;(define value 1) ; this causes an infinite loop somewhere
 ;(openlet (inlet 'i 0 'list-set! (lambda (l . args) (apply #_list-set! l ((car args) 'i) (cdr args))))))
 
-(define-constant constants (list #f #t () #\a (/ most-positive-fixnum) (/ -1 most-positive-fixnum) 1.5+i
+(define-constant auto-constants (list #f #t () #\a (/ most-positive-fixnum) (/ -1 most-positive-fixnum) 1.5+i
 			"hi455" :key hi: 'hi (list 1) (list 1 2) (cons 1 2) (list (list 1 2)) (list (list 1)) (list ()) #() 
 			1/0+i 0+0/0i 0+1/0i 1+0/0i 0/0+0i 0/0+0/0i 1+1/0i 0/0+i cons ''2 
 			1+i 1+1e10i 1e15+1e15i 0+1e18i 1e18 #\xff (string #\xff) 1e308 
 			most-positive-fixnum most-negative-fixnum (- most-positive-fixnum 1) (+ most-negative-fixnum 1)
 			-1 0 0.0 1 1.5 1.0-1.0i 3/4 #\null -63 (make-hash-table) (hash-table '(a . 2) '(b . 3))
-			'((1 2) (3 4)) '((1 (2)) (((3) 4))) "" (list #(1) "1") '(1 2 . 3) (list (cons 'a 2) (cons 'b 3))
-			#(1 2) (vector 1 '(3)) (let ((x 3)) (lambda (y) (+ x y))) abs 'a 'b one
+			'((1 2) (3 4)) '((1 (2)) (((3) 4))) "" (list #i(1) "1") '(1 2 . 3) (list (cons 'a 2) (cons 'b 3))
+			#i(1 2) (vector 1 '(3)) (let ((x 3)) (lambda (y) (+ x y))) abs 'a 'b one
 			(lambda args args) (lambda* ((a 3) (b 2)) (+ a b)) (lambda () 3)
 			(sublet () 'a 1) (rootlet)
 			*load-hook*  *error-hook* (random-state 123)
@@ -49,11 +49,11 @@
 			#<undefined> #<unspecified> (make-int-vector 3) (make-float-vector 3 -1.4)
 			(make-vector '(2 3) "hi") #("hiho" "hi" "hoho") (make-shared-vector (make-int-vector '(2 3) 1) '(6))
 			(make-shared-vector (make-shared-vector (make-float-vector '(2 3) 1.0) '(6)) '(2 2))
-			(vector-ref #2d((#(1 2 3)) (#(3 4 5))) 0 0) (define-macro (m a) `(+ ,a 1))
+			(vector-ref #2d((#i(1 2 3)) (#i(3 4 5))) 0 0) (define-macro (m a) `(+ ,a 1))
 			(c-pointer 0) (c-pointer -1) :readable :else (define-bacro* (m (a 1)) `(+ ,a 1))
 			(byte-vector 0 1 2) (byte-vector) (byte-vector 255 0 127) (make-iterator #((a . 2)))
 			(lambda (dir) 1.0) (float-vector) (make-float-vector '(2 32)) 
-			'((a . 1)) #(1) '((((A . B) C . D) (E . F) G . H) ((I . J) K . L) (M . N) O . P)
+			'((a . 1)) #i(1) '((((A . B) C . D) (E . F) G . H) ((I . J) K . L) (M . N) O . P)
 
 			(mock-number 0) (mock-number 1-i) (mock-number 4/3) (mock-number 2.0)
 			(mock-string #\h #\o #\h #\o)
@@ -65,11 +65,11 @@
 			np mp nv mv ns ms (gensym)
 			))
       
-(define car-constants (car constants))
-(define-constant cdr-constants (cdr constants))
+(define car-auto-constants (car auto-constants))
+(define-constant cdr-auto-constants (cdr auto-constants))
 
 (define low 0)
-(define-constant arglists (vector (make-list 1) (make-list 2) (make-list 3) (make-list 4) (make-list 5) (make-list 6)))
+(define-constant auto-arglists (vector (make-list 1) (make-list 2) (make-list 3) (make-list 4) (make-list 5) (make-list 6)))
 
 (define-constant (autotest func args args-now args-left sig)
   ;; args-left is at least 1, args-now starts at 0, args starts at ()
@@ -88,14 +88,33 @@
 		      (memq (car any) '(wrong-type-arg wrong-number-of-args syntax-error)))
 		 (quit)))))
      
-     (let ((c-args (vector-ref arglists args-now)))
+     (let ((c-args (vector-ref auto-arglists args-now)))
        (copy args c-args)
 
-       (let ((p (list-tail c-args args-now)))
+       (let ((p (list-tail c-args args-now))
+	     (checker (and (pair? sig) (car sig))))
+	 
+	 (define (call-func1 c)
+	   (when (checker c)
+	     (catch #t 
+	       (lambda () 
+		 (set-car! p c)
+		 (apply func c-args))
+	       (lambda any 
+		 'error))))
+
+	 (define (call-func2 c)
+	   (catch #t 
+	     (lambda () 
+	       (set-car! p c)
+	       (apply func c-args))
+	     (lambda any 
+	       'error)))
+
 	 (if (= args-left 1)
 	     (call-with-exit
 	      (lambda (quit)
-		(set-car! p car-constants)
+		(set-car! p car-auto-constants)
 		(catch #t
 		  (lambda ()
 		    (apply func c-args))
@@ -107,41 +126,22 @@
 			     (< (caddr (cadr any)) low))
 			(quit))))
 		 
-		(let ((checker (and (pair? sig) (car sig))))
-		  (if checker
-		      (for-each
-		       (lambda (c)
-			 (when (checker c)
-			   (catch #t 
-			     (lambda () 
-			       (set-car! p c)
-			       (apply func c-args))
-			     (lambda any 
-			       'error))))
-		       cdr-constants)
-		      (for-each
-		       (lambda (c)
-			 (catch #t 
-			   (lambda () 
-			     (set-car! p c)
-			     (apply func c-args))
-			   (lambda any 
-			     'error)))
-		       cdr-constants)))))
+		(if checker ; map-values -> function here
+		    (for-each call-func1 cdr-auto-constants)
+		    (for-each call-func2 cdr-auto-constants))))
 	   
-	     (let ((checker (and (pair? sig) (car sig))))
-	       (if checker
-		   (for-each
-		    (lambda (c)
-		      (when (checker c)
-			(set-car! p c)
-			(autotest func c-args (+ args-now 1) (- args-left 1) (if (pair? sig) (cdr sig) ()))))
-		    constants)
-		   (for-each
-		    (lambda (c)
+	     (if checker
+		 (for-each
+		  (lambda (c)
+		    (when (checker c)
 		      (set-car! p c)
-		      (autotest func c-args (+ args-now 1) (- args-left 1) (if (pair? sig) (cdr sig) ())))
-		    constants)))))))))
+		      (autotest func c-args (+ args-now 1) (- args-left 1) (if (pair? sig) (cdr sig) ()))))
+		  auto-constants)
+		 (for-each
+		  (lambda (c)
+		    (set-car! p c)
+		    (autotest func c-args (+ args-now 1) (- args-left 1) (if (pair? sig) (cdr sig) ())))
+		  auto-constants))))))))
 
 (define safe-fill!
   (let ((signature '(#t sequence? #t)))
@@ -161,21 +161,22 @@
 		  (and (pair? (car lst))
 		       (apply lambda '(x) (list (list 'or (list (caar lst) 'x) (list (cadar lst) 'x)))))))))
 
-(define baddies '(exit emergency-exit abort autotest 
+(define baddies '(exit emergency-exit abort autotest s7-optimize
 		  all delete-file system set-cdr! stacktrace test-sym
 		  cutlet varlet gc cond-expand reader-cond
 		  openlet coverlet eval vector list cons hash-table* hash-table values
-		  throw symbol-table load
+		  symbol-table load local-symbol?
 		  global-environment current-environment make-procedure-with-setter procedure-with-setter? make-rectangular
 		  
-		  copy fill! hash-table-set! vector-set! let-set! list-values apply-values ; omitted { before (see below)
+		  copy fill! hash-table-set! vector-set! let-set! list-values apply-values
 		  
-		  mock-number mock-pair mock-string mock-char mock-vector 
-		  mock-symbol mock-port mock-hash-table m 
+		  mock-number mock-pair mock-string mock-char mock-vector mock-hash-table*
+		  mock-symbol mock-port mock-hash-table m mock->string make-local-method
 		  *mock-number* *mock-pair* *mock-string* *mock-char* *mock-vector*
-		  *mock-symbol* *mock-port* *mock-hash-table*
+		  *mock-symbol* *mock-port* *mock-hash-table* mp ms mv
+		  *unbound-variable-hook* *load-hook* *rootlet-redefinition-hook* *missing-close-paren-hook* *read-error-hook*
 
-		  c-define-1 apropos map-values ;set-current-output-port
+		  c-define-1 apropos map-values
 		  outlet-member make-method make-object))
 
 (define (test-sym sym)
@@ -189,6 +190,7 @@
 		  (strname (symbol->string sym)))
 	      (if (not (memv (strname 0) '(#\{ #\[ #\()))
 		  (begin
+		    ;(if (not (procedure-signature f)) (format *stderr* "~A~%" sym))
 		    (if (< top bottom)
 			(format *stderr* ";~A (bottom: ~A, top: ~A)...~%" sym bottom top))
 					;(format *stderr* ";~A...~%" sym)
@@ -208,9 +210,10 @@
     ;(test-sym 'object->string)
     ;(test-sym 'for-each)
     (format *stderr* "~%all done~%")
-    (s7-version)
     ))
 
 ;(test-sym 'write)
 (all)
+
+(s7-version)
 (exit)
