@@ -42,7 +42,7 @@
 			  (vector-ref #2d((#(1 2 3)) (#(3 4 5))) 0 0) (define-macro (m a) `(+ ,a 1))
 			  (c-pointer 0) (c-pointer -1) :readable *s7* else (define-bacro* (m (a 1)) `(+ ,a 1))
 			  (byte-vector 0 1 2) (byte-vector) (byte-vector 255 0 127) (make-iterator (vector '(a . 2)))
-			  (lambda (dir) 1.0) (float-vector) (make-float-vector '(2 32)) (int-vector 1 2 3) (int-vector)
+			  (lambda (dir) 1.0) (float-vector) (make-float-vector '(2 2)) (int-vector 1 2 3) (int-vector)
 			  (inlet 'value 1 '+ (lambda args 1)) (inlet) (make-iterator (inlet 'a 1 'b 2) (cons #f #f))
 			  (make-iterator "123456") (make-iterator '(1 2 3)) (make-iterator (hash-table* 'a 1 'b 2) (cons #f #f))
 			  (open-input-string "123123") (open-input-file "/home/bil/cl/4.aiff")
@@ -56,7 +56,7 @@
 			  (mock-vector 1 2 3 4)
 			  (mock-hash-table* 'b 2)
 			  
-			  (make-block 32) (block) (make-iterator (block 1 2 3 4 5 6))
+			  (make-block 4) (block) (make-iterator (block 1 2 3 4))
 			  ))
 (define-constant constants-len (length constants))
 
@@ -74,53 +74,46 @@
 (define (test-chars)
   (let ((op (open-output-string))
 	(x #f) (y #f) (z #f))
-    (do ((size 2 (+ size 1)))
+    (do ((size 2 (+ size 1))
+	 (size1 3 (+ size1 1))
+	 (tries 4000 (+ tries 2000))
+	 (pos 0 0)
+	 (ctrl-str ""))
 	((= size 15))
-      (let ((tries (* size 2000))
-	    (size1 (+ size 1)))
-	(format *stderr* "~D " size)
-	(let ((ctrl-str (make-string size1))
-	      (pos 0))
-	  (string-set! ctrl-str 0 #\~)
-	  (do ((i 0 (+ i 1)))
-	      ((= i tries))
-	    (do ((j 1 (+ j 1)))
-		((= j size1))
-	      (string-set! ctrl-str j (string-ref ctrl-chars (random ctrl-chars-len))))
-	    
-	    (set! x (constants (random constants-len)))
-	    (set! y (constants (random constants-len)))
-	    (set! z (constants (random constants-len)))
-	    
-	    (object->string x)
-	    (display x op)
-	    
-	    (catch #t (lambda () (format #f "~{~^~S ~} ~{~|~S ~} ~W" x y z)) (lambda arg 'error))
-	    (catch #t (lambda () (format #f ctrl-str)) (lambda arg 'error))
-	    (catch #t (lambda () (format #f ctrl-str x)) (lambda arg 'error))
-	    (catch #t (lambda () (format #f ctrl-str y)) (lambda arg 'error))
-	    (catch #t (lambda () (format #f ctrl-str z)) (lambda arg 'error))
-	    (set! pos (char-position #\~ ctrl-str 1))
-	    (when pos
-	      (catch #t (lambda () (format #f ctrl-str x z)) (lambda arg 'error))
-	      (catch #t (lambda () (format #f ctrl-str x y)) (lambda arg 'error))
-	      (catch #t (lambda () (format #f ctrl-str y z)) (lambda arg 'error))
-	      (catch #t (lambda () (format #f ctrl-str z x)) (lambda arg 'error))
-	      (when (char-position #\~ ctrl-str (+ pos 1))
-		(catch #t (lambda () (format #f ctrl-str x y z)) (lambda arg 'error))
-		(catch #t (lambda () (format #f ctrl-str z y x)) (lambda arg 'error))))))
-	(get-output-string op #t)))
+      (format *stderr* "~D " size)
+      (set! ctrl-str (make-string size1))
+      (string-set! ctrl-str 0 #\~)
+      (do ((i 0 (+ i 1)))
+	  ((= i tries))
+	(do ((j 1 (+ j 1)))
+	    ((= j size1))
+	  (string-set! ctrl-str j (string-ref ctrl-chars (random ctrl-chars-len))))
+	
+	(set! x (constants (random constants-len)))
+	(set! y (constants (random constants-len)))
+	(set! z (constants (random constants-len)))
+	
+	(object->string x)
+	(display x op)
+	
+	(catch #t (lambda () (format #f "~{~^~S ~} ~{~|~S ~} ~W" x y z)) (lambda arg 'error))
+	(catch #t (lambda () (format #f ctrl-str)) (lambda arg 'error))
+	(catch #t (lambda () (format #f ctrl-str x)) (lambda arg 'error))
+	(catch #t (lambda () (format #f ctrl-str y)) (lambda arg 'error))
+	(catch #t (lambda () (format #f ctrl-str z)) (lambda arg 'error))
+	(set! pos (char-position #\~ ctrl-str 1))
+	(when pos
+	  (catch #t (lambda () (format #f ctrl-str x z)) (lambda arg 'error))
+	  (catch #t (lambda () (format #f ctrl-str x y)) (lambda arg 'error))
+	  (catch #t (lambda () (format #f ctrl-str y z)) (lambda arg 'error))
+	  (catch #t (lambda () (format #f ctrl-str z x)) (lambda arg 'error))
+	  (when (char-position #\~ ctrl-str (+ pos 1))
+	    (catch #t (lambda () (format #f ctrl-str x y z)) (lambda arg 'error))
+	    (catch #t (lambda () (format #f ctrl-str z y x)) (lambda arg 'error)))))
+      (get-output-string op #t))
     (close-output-port op)))
 
 (test-chars)
 
 (s7-version)
-
 (exit)
-
-#|
-valgrind --vgdb=yes --vgdb-error=0 repl
-gdb repl
- target remote | /usr/local/lib/valgrind/../../bin/vgdb
- continue
-|#
