@@ -180,69 +180,78 @@
 (define-constant F (make-env (float-vector 0.0 .1 1.0 1.0) :length 100))
 (define-constant K (float-vector 0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0))
 (define-constant V (make-float-vector 10))
+(define-constant VV1 (make-float-vector 10))
+(define-constant VV4 (make-float-vector 10))
+(define-constant VV5 (make-float-vector 10))
+(define-constant VV6 (make-float-vector 10))
+(define-constant VV8 (make-float-vector 10))
+(define-constant VV10 (make-float-vector 10))
+(define (Z) (mus-copy F))
+
+(define G #f)
+(define I #f)
+(define (O) (vector #f (mus-copy I) #f))
+(define (Q) (mus-copy G))
 
 (define (try1 form gen)
   (let ((make-gen (string->symbol (string-append "make-" (symbol->string gen)))))
     (let ((body
      `(let ()
-	(define G (,make-gen 1000)) 
-	(define I (,make-gen 500)) 
-	(define (O) (vector #f (mus-copy I) #f))
-	(define (Q) (mus-copy G))
-	(define (Z) (mus-copy F))
-
-	(let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
-	  (do ((i 0 (+ i 1))) ((= i 10))
-	    (set! (V i) ,(copy form))))
-	
 	(define (tester-1)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v VV1))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (float-vector-set! v i ,(copy form)))))
+	      (float-vector-set! v i ,form))))
 	
 	(define (tester-2)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (outa i ,(copy form)))))
+	      (outa i ,form))))
 	
 	(define (tester-3)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (out-any i ,(copy form) 0))))
+	      (out-any i ,form 0))))
 	
 	(define (tester-4)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (do ((i 0 (+ i 1)) (lst ())) ((= i 10) (apply float-vector (reverse! lst)))
-	      (set! lst (cons ,(copy form) lst)))))
+	      (set! lst (cons ,form lst)))))
 	
 	(define (tester-5)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v VV4))
 	    (set! *output* (make-sample->file "test.snd" 1 mus-ldouble mus-next "t816"))
 	    (do ((i 0 (+ i 1)) (x 0.0 (+ x 0.1))) ((= i 10))
-	      (outa i ,(copy form)))
+	      (outa i ,form))
 	    (mus-close *output*)
 	    (file->array "test.snd" 0 0 10 v)))
 	
 	(define (tester-6)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (k 1) (v VV5))
 	    (do ((i 0 (+ i 1)) (y -0.5) (x 0.0 (+ x 0.1))) ((= i 10) v)
-	      (float-vector-set! v i ,(copy form)))))
+	      (float-vector-set! v i ,form))))
 	
 	(define (tester-11)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v VV6))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
 	      (let ((x (,gen o)))
-		(set! (v i) ,(copy form))))))
+		(set! (v i) ,form)))))
 	
 	(define (tester-12)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
 	      (let ((x (,gen o)))
-		(outa i ,(copy form))))))
+		(outa i ,form)))))
 
+	(set! G (,make-gen 1000)) 
+	(set! I (,make-gen 500)) 
+
+	(let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
+	  (do ((i 0 (+ i 1))) ((= i 10))
+	    (set! (V i) ,form)))
+	
 	(checkout-1 ',form V (tester-1) (tester-2) (tester-3) (tester-4) (tester-5) (tester-6) (tester-11) (tester-12))
 	)))
       (define the-body (apply lambda () (list (copy body :readable))))
@@ -252,95 +261,91 @@
   (let ((make-gen (string->symbol (string-append "make-" (symbol->string gen)))))
     (let ((body
      `(let ()
-	(define G (,make-gen 1000)) 
-	(define I (,make-gen 500)) 
-	(define (O) (vector #f (mus-copy I) #f))
-	(define (Q) (mus-copy G))
-	(define (Z) (mus-copy F))
-
-	(let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
-	  (do ((i 0 (+ i 1))) ((= i 10))
-	    (set! (V i) ,(copy form))))
-	
 	(define (tester-1)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v VV1))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (float-vector-set! v i ,(copy form)))))
+	      (float-vector-set! v i ,form))))
 	
 	(define (tester-2)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (outa i ,(copy form)))))
+	      (outa i ,form))))
 	
 	(define (tester-3)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (out-any i ,(copy form) 0))))
+	      (out-any i ,form 0))))
 	
 	(define (tester-4)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (do ((i 0 (+ i 1)) (lst ())) ((= i 10) (apply float-vector (reverse! lst)))
-	      (set! lst (cons ,(copy form) lst)))))
+	      (set! lst (cons ,form lst)))))
 	
 	(define (tester-5)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v VV4))
 	    (set! *output* (make-sample->file "test.snd" 1 mus-ldouble mus-next "t816"))
 	    (do ((i 0 (+ i 1)) (x 0.0 (+ x 0.1))) ((= i 10))
-	      (outa i ,(copy form)))
+	      (outa i ,form))
 	    (mus-close *output*)
 	    (file->array "test.snd" 0 0 10 v)))
 	
 	(define (tester-6)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (k 1) (v VV5))
 	    (do ((i 0 (+ i 1)) (y -0.5) (x 0.0 (+ x 0.1))) ((= i 10) v)
-	      (float-vector-set! v i ,(copy form)))))
+	      (float-vector-set! v i ,form))))
 	
 	(define (tester-7)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v VV6))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (let ((z ,(copy form)))
+	      (let ((z ,form))
 		(float-vector-set! v i (,gen o z))))))
 	
 	(define (tester-8)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (let ((z ,(copy form)))
+	      (let ((z ,form))
 		(outa i (,gen o z))))))
 	
 	(define (tester-9)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1) (v VV8))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (let ((z ,(copy form)))
+	      (let ((z ,form))
 		(float-vector-set! v i (* (env a) (,gen o z)))))))
 	
 	(define (tester-10)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (let ((z ,(copy form)))
+	      (let ((z ,form))
 		(outa i (* (env a) (,gen o z)))))))
 	
 	(define (tester-11)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1) (v VV10))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
 	      (let ((x (,gen o)))
-		(set! (v i) ,(copy form))))))
+		(set! (v i) ,form)))))
 	
 	(define (tester-12)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (y -0.5) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
 	      (let ((x (,gen o)))
-		(outa i ,(copy form))))))
+		(outa i ,form)))))
 
+	(set! G (,make-gen 1000)) 
+	(set! I (,make-gen 500)) 
+
+	(let ((o (Q)) (p (Q)) (q (Q)) (oscs (O)) (a (Z)) (b (Z)) (x 3.14) (y -0.5) (k 1))
+	  (do ((i 0 (+ i 1))) ((= i 10))
+	    (set! (V i) ,form)))
+	
 	(checkout ',form V
 		  (tester-1) (tester-2) (tester-3) (tester-4) (tester-5) (tester-6) 
 		  (tester-7) (tester-8) (tester-9) (tester-10) (tester-11) (tester-12))
 	)))
-
       (define the-body (apply lambda () (list (copy body :readable))))
       (the-body))))
 
@@ -348,97 +353,93 @@
   (let ((make-gen (string->symbol (string-append "make-" (symbol->string gen)))))
     (let ((body
      `(let ()
-	(define G (,make-gen 1000)) 
-	(define I (,make-gen 500)) 
-	(define (O) (vector #f (mus-copy I) #f))
-	(define (Q) (mus-copy G))
-	(define (Z) (mus-copy F))
-
-	(let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
-	  (do ((i 0 (+ i 1))) ((= i 10))
-	    (float-vector-set! V i ,(copy form))))
-	
 	(define (tester-1)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1) (v VV1))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (float-vector-set! v i ,(copy form)))))
+	      (float-vector-set! v i ,form))))
 	
 	(define (tester-2)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (outa i ,(copy form)))))
+	      (outa i ,form))))
 	
 	(define (tester-3)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (out-any i ,(copy form) 0))))
+	      (out-any i ,form 0))))
 	
 	(define (tester-4)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (k 1) (z 0.1))
 	    (do ((i 0 (+ i 1)) (lst ())) ((= i 10) (apply float-vector (reverse! lst)))
-	      (set! lst (cons ,(copy form) lst)))))
+	      (set! lst (cons ,form lst)))))
 	
 	(define (tester-5)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (y -0.5) (k 1) (z 0.1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (y -0.5) (k 1) (z 0.1) (v VV4))
 	    (set! *output* (make-sample->file "test.snd" 1 mus-ldouble mus-next "t816"))
 	    (do ((i 0 (+ i 1)) (x 0.0 (+ x 0.1))) ((= i 10))
-	      (outa i ,(copy form)))
+	      (outa i ,form))
 	    (mus-close *output*)
 	    (file->array "test.snd" 0 0 10 v)))
 	
 	(define (tester-6)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (k 1) (v VV5))
 	    (do ((i 0 (+ i 1)) (y -0.5) (z 0.1) (x 0.0 (+ x 0.1))) ((= i 10) v)
-	      (set! (v i) ,(copy form)))))
+	      (set! (v i) ,form))))
 	
 	(define (tester-7)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (k 1) (z 0.1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (k 1) (z 0.1) (v VV6))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (let ((zz ,(copy form)))
+	      (let ((zz ,form))
 		(float-vector-set! v i (,gen o zz))))))
 	
 	(define (tester-8)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (let ((zz ,(copy form)))
+	      (let ((zz ,form))
 		(outa i (,gen o zz))))))
 	
 	(define (tester-9)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1) (v VV8))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
-	      (let ((zz ,(copy form)))
+	      (let ((zz ,form))
 		(float-vector-set! v i (* (env a) (,gen o zz)))))))
 	
 	(define (tester-10)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
-	      (let ((zz ,(copy form)))
+	      (let ((zz ,form))
 		(outa i (* (env a) (,gen o zz)))))))
 	
 	(define (tester-11)
-	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (y -0.5) (z 0.1) (k 1) (v (make-float-vector 10)))
+	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (y -0.5) (z 0.1) (k 1) (v VV10))
 	    (do ((i 0 (+ i 1))) ((= i 10) v)
 	      (let ((x (,gen o)))
-		(float-vector-set! v i ,(copy form))))))
+		(float-vector-set! v i ,form)))))
 	
 	(define (tester-12)
 	  (let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (y -0.5) (z 0.1) (k 1))
 	    (set! *output* (make-float-vector 10))
 	    (do ((i 0 (+ i 1))) ((= i 10) *output*)
 	      (let ((x (,gen o)))
-		(outa i ,(copy form))))))
+		(outa i ,form)))))
 
+	(set! G (,make-gen 1000)) 
+	(set! I (,make-gen 500)) 
+
+	(let ((o (Q)) (p (Q)) (q (Q)) (s (Q)) (t (Q)) (oscs (O)) (a (Z)) (b (Z)) (c (Z)) (d (Z)) (x 3.14) (y -0.5) (z 0.1) (k 1))
+	  (do ((i 0 (+ i 1))) ((= i 10))
+	    (float-vector-set! V i ,form)))
+	
 	(if (= args 1)
 	    (checkout-1 ',form V (tester-1) (tester-2) (tester-3) (tester-4) (tester-5) (tester-6) (tester-11) (tester-12))
 	    (checkout ',form V
 		      (tester-1) (tester-2) (tester-3) (tester-4) (tester-5) (tester-6) 
 		      (tester-7) (tester-8) (tester-9) (tester-10) (tester-11) (tester-12))
 	    ))))
-
       (define the-body (apply lambda () (list (copy body :readable))))
       (the-body))))
 
@@ -707,5 +708,7 @@
  )
 
 ;(gc)
+
 (s7-version)
 (exit)
+
