@@ -1,16 +1,22 @@
-(define file-names '(("teq.scm" . "v-eq")
-		     ("titer.scm" . "v-iter")
-		     ("tmap.scm" . "v-map")
-		     ("tform.scm" . "v-form")
-		     ("thash.scm" . "v-hash")
-		     ("tcopy.scm" . "v-cop")
-		     ;("lg.scm" . "v-lg")
-		     ("tgen.scm" . "v-gen")
+(define file-names '(("make-index.scm" . "v-index")
+		     ("tmac.scm" . "v-mac")
+		     ("teq.scm" . "v-eq")
+		     ("tfft.scm" . "v-fft")
+		     ("tref.scm" . "v-ref")
+		     ("tlet.scm" . "v-let")
 		     ("tauto.scm" . "v-auto")
-		     ("make-index.scm" . "v-index")
-		     ("snd-test.scm" . "v-call")
-		     ("tall.scm" . "v-all")
 		     ("s7test.scm" . "v-test")
+		     ("tcopy.scm" . "v-cop")
+		     ("lt.scm" . "v-lt")
+		     ("tform.scm" . "v-form")
+		     ("tmap.scm" . "v-map")
+		     ;("lg.scm" . "v-lg")
+		     ("titer.scm" . "v-iter")
+		     ("tsort.scm" . "v-sort")
+		     ("thash.scm" . "v-hash")
+		     ("tgen.scm" . "v-gen")
+		     ("tall.scm" . "v-all")
+		     ("snd-test.scm" . "v-call")
 		     ))
 
 (define (last-callg)
@@ -35,19 +41,32 @@
    (lambda (caller+file)
      (system "rm callg*")
      (format *stderr* "~%~NC~%~NC ~A ~NC~%~NC~%" 40 #\- 16 #\- (cadr caller+file) 16 #\- 40 #\-)
-     (system (format #f "valgrind --tool=callgrind ./~A ~A" (car caller+file) (cadr caller+file)))
+     (system (format #f "/home/bil/test/valgrind-3.12.0/vg-in-place --tool=callgrind ./~A ~A" (car caller+file) (cadr caller+file)))
+     ;; new valgrind blathers endlessly -- I made this change:
+     ;;   /home/bil/test/valgrind-3.12.0/coregrind/m_syswrap/syswrap-generic.c
+     ;;   comment out lines 1333 to 1341
      (let ((outfile (cdr (assoc (cadr caller+file) file-names))))
        (let ((next (next-file outfile)))
-	 (system (format #f "callgrind_annotate --auto=yes --threshold=100 ~A > ~A~D" (last-callg) outfile next))
+	 (system (format #f "/home/bil/test/valgrind-3.12.0/callgrind/callgrind_annotate --auto=yes --threshold=100 ~A > ~A~D" (last-callg) outfile next))
+	 ;; new callgrind blathers endlessly -- I made this change:
+         ;;   (line 825) my $space = ' ' x ($CC_col_widths->[$i] - length($count));
+         ;;              my $space = ' ' x max($CC_col_widths->[$i] - length($count), 0);
 	 (format *stderr* "~NC ~A~D -> ~A~D: ~NC~%" 8 #\space outfile (- next 1) outfile next 8 #\space)
-	 (system (format #f "./snd compare-calls.scm -e '(compare-calls \"~A~D\" \"~A~D\")'" outfile (- next 1) outfile next)))))
-   (list (list "repl" "teq.scm")
-	 (list "repl" "s7test.scm")
+	 (system (format #f "./snd compare-calls.scm -e '(compare-calls \"~A~D\" \"~A~D\")'" outfile (- next 1) outfile next))))
+     (system (format #f "/home/bil/test/valgrind-3.12.0/vg-in-place ./~A ~A" (car caller+file) (cadr caller+file))))
+   (list (list "repl" "tmac.scm")
 	 (list "snd -noinit" "make-index.scm")
-	 (list "repl" "tmap.scm")
-	 (list "repl" "tform.scm")
-	 (list "repl" "tcopy.scm")
+	 (list "repl" "tref.scm")
 	 (list "repl" "tauto.scm")
+	 (list "repl" "teq.scm")
+	 (list "repl" "s7test.scm")
+	 (list "repl" "tlet.scm")
+	 (list "repl" "lt.scm")
+	 (list "repl" "tcopy.scm")
+	 (list "repl" "tform.scm")
+	 (list "repl" "tmap.scm")
+	 (list "repl" "tfft.scm")
+	 (list "repl" "tsort.scm")
 	 (list "repl" "titer.scm")
 	 ;(list "repl" "lg.scm")
 	 (list "repl" "thash.scm")
@@ -64,4 +83,4 @@
 (load "compare-calls.scm")
 (combine-latest)
 
-(exit)
\ No newline at end of file
+(exit)
