@@ -59,6 +59,12 @@
   #pragma warning(disable: 4244)
 #endif
 
+#if (defined(__GNUC__) && __GNUC__ >= 5)
+  #define WITH_VECTORIZE 1
+#else
+  #define WITH_VECTORIZE 0
+#endif
+
 #include "_sndlib.h"
 #include "xen.h"
 #include "clm.h"
@@ -78,6 +84,7 @@ mus_float_t *mus_vct_data(vct *v) {return(v->data);}
 #endif
 
 #if HAVE_SCHEME
+
 #define S_make_vct       "make-float-vector"
 #define S_vct_add        "float-vector-add!"
 #define S_vct_subtract   "float-vector-subtract!"
@@ -154,6 +161,34 @@ mus_float_t *mus_vct_data(vct *v) {return(v->data);}
   #define VCT_PRINT_LENGTH 10
 #endif
 
+
+#if WITH_VECTORIZE
+void mus_clear_floats(mus_float_t *dst, mus_long_t len)
+{
+  mus_long_t k;
+  for (k = 0; k < len; k++) dst[k] = 0.0;
+}
+
+void mus_copy_floats(mus_float_t *dst, mus_float_t *src, mus_long_t len)
+{
+  mus_long_t k;
+  for (k = 0; k < len; k++) dst[k] = src[k];
+}
+
+void mus_add_floats(mus_float_t *dst, mus_float_t *src, mus_long_t len)
+{
+  mus_long_t k;
+  for (k = 0; k < len; k++) dst[k] += src[k];
+}
+
+void mus_abs_floats(mus_float_t *dst, mus_long_t len)
+{
+  mus_long_t k;
+  for (k = 0; k < len; k++) dst[k] = fabs(dst[k]);
+}
+#endif
+
+
 static int vct_print_length = VCT_PRINT_LENGTH;
 
 void mus_vct_set_print_length(int len) 
@@ -442,7 +477,7 @@ static Xen g_vct_copy(Xen obj)
   if (len > 0)
     {
       copied_data = (mus_float_t *)malloc(len * sizeof(mus_float_t));
-      memcpy((void *)copied_data, (void *)(mus_vct_data(v)), (len * sizeof(mus_float_t)));
+      mus_copy_floats(copied_data, mus_vct_data(v), len);
     }
   return(xen_make_vct(len, copied_data));
 }
@@ -594,6 +629,13 @@ static Xen g_vct_multiply(Xen obj1, Xen obj2)
   return(obj1);
 }
 
+#if WITH_VECTORIZE
+static void vct_add(mus_float_t *d1, mus_float_t *d2, mus_long_t lim) __attribute__((optimize("tree-vectorize")));
+static void vct_add(mus_float_t *d1, mus_float_t *d2, mus_long_t lim)
+{
+  mus_add_floats(d1, d2, lim);
+}
+#else
 static void vct_add(mus_float_t *d1, mus_float_t *d2, mus_long_t lim)
 {
   mus_long_t i, lim8;
@@ -621,6 +663,7 @@ static void vct_add(mus_float_t *d1, mus_float_t *d2, mus_long_t lim)
   for (; i < lim; i++) 
     d1[i] += d2[i];
 }
+#endif
 
 static Xen g_vct_add(Xen obj1, Xen obj2, Xen offs)
 {
@@ -698,7 +741,7 @@ static Xen g_vct_subtract(Xen obj1, Xen obj2)
 static Xen g_vct_abs(Xen obj)
 {
   #define H_vct_absB "(" S_vct_abs " v): v[i] = abs(v[i]), return v."
-  mus_long_t i, lim;
+  mus_long_t lim;
   vct *v;
   mus_float_t *d;
 
@@ -707,7 +750,7 @@ static Xen g_vct_abs(Xen obj)
   v = Xen_to_vct(obj);
   d = mus_vct_data(v);
   lim = mus_vct_length(v);
-  for (i = 0; i < lim; i++) d[i] = fabs(d[i]);
+  mus_abs_floats(d, lim);
   return(obj);
 }
 
@@ -755,10 +798,14 @@ static Xen g_vct_equal(Xen uv1, Xen uv2, Xen udiff)
   return(C_double_to_Xen_real(max_diff));
 }
 
+#if WITH_VECTORIZE
+static void vct_scale(mus_float_t *d, mus_float_t scl, mus_long_t len) __attribute__((optimize("tree-vectorize")));
+#endif
+
 static void vct_scale(mus_float_t *d, mus_float_t scl, mus_long_t len)
 {
   if (scl == 0.0)
-    memset((void *)d, 0, len * sizeof(mus_float_t));
+    mus_clear_floats(d, len);
   else
     {
       if (scl != 1.0)
@@ -804,6 +851,7 @@ static Xen g_vct_offset(Xen obj1, Xen obj2)
   vct *v1;
   mus_float_t scl;
   mus_float_t *d;
+  mus_long_t len;
 
   Xen_check_type(mus_is_vct(obj1), obj1, 1, S_vct_offset, A_VCT);
   Xen_check_type(Xen_is_number(obj2), obj2, 2, S_vct_offset, "a number");
@@ -811,12 +859,13 @@ static Xen g_vct_offset(Xen obj1, Xen obj2)
   v1 = Xen_to_vct(obj1);
   if (mus_vct_length(v1) == 0) return(obj1);
   d = mus_vct_data(v1);
+  len = mus_vct_length(v1);
 
   scl = Xen_real_to_C_double(obj2);
   if (scl != 0.0)
     {
       mus_long_t i;
-      for (i = 0; i < mus_vct_length(v1); i++) 
+      for (i = 0; i < len; i++) 
 	d[i] += scl;
     }
   return(obj1);
@@ -882,7 +931,7 @@ index0 and index1 interpolating between x2 and x1 by incrementing x0 by dx"
 static Xen g_vct_fill(Xen obj1, Xen obj2)
 {
   #define H_vct_fillB "(" S_vct_fill " v val): set each element of v to val: v[i] = val, returns v"
-  mus_long_t i; /* unsigned int is much slower */
+  mus_long_t i, len; /* unsigned int is much slower */
   vct *v1;
   mus_float_t scl;
   mus_float_t *d;
@@ -891,16 +940,17 @@ static Xen g_vct_fill(Xen obj1, Xen obj2)
   Xen_check_type(Xen_is_number(obj2), obj2, 2, S_vct_fill, "a number");
 
   v1 = Xen_to_vct(obj1);
-  if (mus_vct_length(v1) == 0) return(obj1);
+  len = mus_vct_length(v1);
+  if (len == 0) return(obj1);
   d = mus_vct_data(v1);
 
   scl = Xen_real_to_C_double(obj2);
   if (scl == 0.0)
-    memset((void *)d, 0, mus_vct_length(v1) * sizeof(mus_float_t));
+    mus_clear_floats(d, len);
   else 
     {
       mus_long_t lim8;
-      lim8 = mus_vct_length(v1) - 8;
+      lim8 = len - 8;
       i = 0;
       while (i <= lim8)
 	{
@@ -913,45 +963,25 @@ static Xen g_vct_fill(Xen obj1, Xen obj2)
 	  d[i++] = scl;
 	  d[i++] = scl;
 	}
-      for (; i < mus_vct_length(v1); i++) 
+      for (; i < len; i++) 
 	d[i] = scl;
     }
   return(obj1);
 }
 #endif
 
-
 double mus_vct_peak(vct *v)
 {
-  mus_float_t val = 0.0, absv;
+  mus_float_t val = 0.0;
   mus_float_t *d;
-  mus_long_t i, lim4, len;
+  mus_long_t i, len;
 
   len = mus_vct_length(v);
-
   if (len == 0) return(0.0);
-  lim4 = len - 4;
-  i = 1;
   d = mus_vct_data(v);
-  val = fabs(d[0]);
 
-  while (i <= lim4)
-    {
-      absv = fabs(d[i++]);
-      if (absv > val) val = absv;
-      absv = fabs(d[i++]);
-      if (absv > val) val = absv;
-      absv = fabs(d[i++]);
-      if (absv > val) val = absv;
-      absv = fabs(d[i++]);
-      if (absv > val) val = absv;
-    }
-
-  for (; i < len; i++)
-    {
-      absv = fabs(d[i]);
-      if (absv > val) val = absv;
-    }
+  for (i = 0; i < len; i++)
+    val = (fabs(d[i]) > val) ? fabs(d[i]) : val;
   return(val);
 }
 
@@ -970,19 +1000,21 @@ Xen g_vct_peak(Xen obj)
 #define S_vct_peak_and_location "vct-peak-and-location"
 #endif
 
+
 static Xen g_vct_peak_and_location(Xen obj)
 {
   #define H_vct_peak_and_location "(" S_vct_peak_and_location " v): max of abs of elements of v and its position in v"
   mus_float_t val = 0.0;
-  mus_long_t i, loc = 0;
+  mus_long_t i, loc = 0, len;
   vct *v;
   mus_float_t *d;
 
   Xen_check_type(mus_is_vct(obj), obj, 1, S_vct_peak_and_location, "a " S_vct);
   v = Xen_to_vct(obj);
   d = mus_vct_data(v);
+  len = mus_vct_length(v);
 
-  for (i = 0; i < mus_vct_length(v); i++)
+  for (i = 0; i < len; i++)
     {
       mus_float_t absv;
       absv = fabs(d[i]);
@@ -1212,6 +1244,13 @@ static mus_float_t vct_max(mus_float_t *d, mus_long_t len)
   return(mx);
 }
 
+#if HAVE_SCHEME
+static s7_double float_vector_max_d_p(s7_pointer v)
+{
+  return(vct_max(s7_float_vector_elements(v), s7_vector_length(v)));
+}
+#endif
+
 static Xen g_vct_max(Xen vobj)
 {
   #define H_vct_max "(" S_vct_max " v): returns the maximum element of " S_vct
@@ -1239,6 +1278,13 @@ static mus_float_t vct_min(mus_float_t *d, mus_long_t len)
   return(mx);
 }
 
+#if HAVE_SCHEME
+static s7_double float_vector_min_d_p(s7_pointer v)
+{
+  return(vct_min(s7_float_vector_elements(v), s7_vector_length(v)));
+}
+#endif
+
 static Xen g_vct_min(Xen vobj)
 {
   #define H_vct_min "(" S_vct_min " v): returns the minimum element of " S_vct
@@ -1541,200 +1587,6 @@ vct( 0.5 0.3 0.1 ) .g => #<vct[len=3]: 0.500 0.300 0.100>"
 #endif
 
 
-#if HAVE_SCHEME
-
-#define PF_TO_RF(CName, Cfnc)					  \
-  static s7_double CName ## _rf_a(s7_scheme *sc, s7_pointer **p) \
-  {								  \
-    s7_pf_t f;							  \
-    s7_pointer x;						  \
-    f = (s7_pf_t)(**p); (*p)++;					  \
-    x = f(sc, p);						  \
-    return(Cfnc);						  \
-  }								  \
-  static s7_rf_t CName ## _rf(s7_scheme *sc, s7_pointer expr)	  \
-  {									\
-    if ((s7_is_pair(s7_cdr(expr))) && (s7_is_null(sc, s7_cddr(expr))) && \
-        (s7_arg_to_pf(sc, s7_cadr(expr))))				\
-      return(CName ## _rf_a);						\
-    return(NULL);							\
-  }
-
-static s7_double c_vct_max(s7_scheme *sc, s7_pointer x)
-{
-  s7_int len;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_max, 1, x, "a float-vector");
-  len = s7_vector_length(x);
-  if (len == 0) return(0.0);
-  return(vct_max(s7_float_vector_elements(x), len));
-}
-
-PF_TO_RF(float_vector_max, c_vct_max(sc, x))
-
-static s7_double c_vct_min(s7_scheme *sc, s7_pointer x)
-{
-  s7_int len;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_min, 1, x, "a float-vector");
-  len = s7_vector_length(x);
-  if (len == 0) return(0.0);
-  return(vct_min(s7_float_vector_elements(x), len));
-}
-
-PF_TO_RF(float_vector_min, c_vct_min(sc, x))
-
-PF_TO_RF(float_vector_peak, mus_vct_peak(x))
-
-
-
-#define PF2_TO_PF(CName, Cfnc)					  \
-  static s7_pointer CName ## _pf_a(s7_scheme *sc, s7_pointer **p) \
-  {								  \
-    s7_pf_t f;							  \
-    s7_pointer x, y;						  \
-    f = (s7_pf_t)(**p); (*p)++;					  \
-    x = f(sc, p);						  \
-    f = (s7_pf_t)(**p); (*p)++;					  \
-    y = f(sc, p);						  \
-    return(Cfnc);						  \
-  }								  \
-  static s7_pf_t CName ## _pf(s7_scheme *sc, s7_pointer expr)	  \
-  {									\
-    if ((s7_is_pair(s7_cdr(expr))) && (s7_is_pair(s7_cddr(expr))) && (s7_is_null(sc, s7_cdddr(expr))) && \
-        (s7_arg_to_pf(sc, s7_cadr(expr))) &&				\
-        (s7_arg_to_pf(sc, s7_caddr(expr))))				\
-      return(CName ## _pf_a);						\
-    return(NULL);							\
-  }
-
-static s7_pointer c_vct_add(s7_scheme *sc, s7_pointer x, s7_pointer y)
-{
-  s7_int len1, lim; 
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_add, 1, x, "a float-vector");
-  if (!s7_is_float_vector(y)) s7_wrong_type_arg_error(sc, S_vct_add, 2, y, "a float-vector");
-  len1 = s7_vector_length(x);
-  lim = s7_vector_length(y);
-  if (lim > len1) lim = len1;
-  if (lim == 0) return(x);
-  vct_add(s7_float_vector_elements(x), s7_float_vector_elements(y), lim);
-  return(x);
-}
-
-PF2_TO_PF(float_vector_add, c_vct_add(sc, x, y))
-
-static s7_pointer c_vct_subtract(s7_scheme *sc, s7_pointer x, s7_pointer y)
-{
-  s7_int i, len1, lim; 
-  s7_double *fx, *fy;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_subtract, 1, x, "a float-vector");
-  if (!s7_is_float_vector(y)) s7_wrong_type_arg_error(sc, S_vct_subtract, 2, y, "a float-vector");
-  len1 = s7_vector_length(x);
-  lim = s7_vector_length(y);
-  if (lim > len1) lim = len1;
-  if (lim == 0) return(x);
-  fx = s7_float_vector_elements(x);
-  fy = s7_float_vector_elements(y);
-  for (i = 0; i < lim; i++) fx[i] -= fy[i];
-  return(x);
-}
-
-PF2_TO_PF(float_vector_subtract, c_vct_subtract(sc, x, y))
-
-static s7_pointer c_vct_multiply(s7_scheme *sc, s7_pointer x, s7_pointer y)
-{
-  s7_int i, len1, lim; 
-  s7_double *fx, *fy;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_multiply, 1, x, "a float-vector");
-  if (!s7_is_float_vector(y)) s7_wrong_type_arg_error(sc, S_vct_multiply, 2, y, "a float-vector");
-  len1 = s7_vector_length(x);
-  lim = s7_vector_length(y);
-  if (lim > len1) lim = len1;
-  if (lim == 0) return(x);
-  fx = s7_float_vector_elements(x);
-  fy = s7_float_vector_elements(y);
-  for (i = 0; i < lim; i++) fx[i] *= fy[i];
-  return(x);
-}
-
-PF2_TO_PF(float_vector_multiply, c_vct_multiply(sc, x, y))
-
-#define PRF_TO_PF(CName, Cfnc)					  \
-  static s7_pointer CName ## _pf_a(s7_scheme *sc, s7_pointer **p) \
-  {								  \
-    s7_pf_t f;							  \
-    s7_rf_t r;							  \
-    s7_pointer x;						  \
-    s7_double y;						  \
-    f = (s7_pf_t)(**p); (*p)++;					  \
-    x = f(sc, p);						  \
-    r = (s7_rf_t)(**p); (*p)++;					  \
-    y = r(sc, p);						  \
-    return(Cfnc);						  \
-  }								  \
-  static s7_pf_t CName ## _pf(s7_scheme *sc, s7_pointer expr)	  \
-  {									\
-    if ((s7_is_pair(s7_cdr(expr))) && (s7_is_pair(s7_cddr(expr))) && (s7_is_null(sc, s7_cdddr(expr))) && \
-        (s7_arg_to_pf(sc, s7_cadr(expr))) &&				\
-        (s7_arg_to_rf(sc, s7_caddr(expr))))				\
-      return(CName ## _pf_a);						\
-    return(NULL);							\
-  }
-
-
-static s7_pointer c_vct_scale(s7_scheme *sc, s7_pointer x, s7_double y)
-{
-  s7_int len;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_scale, 1, x, "a float-vector");
-  len = s7_vector_length(x);
-  if (len == 0) return(x);
-  vct_scale(s7_float_vector_elements(x), y, len);
-  return(x);
-}
-
-PRF_TO_PF(float_vector_scale, c_vct_scale(sc, x, y))
-
-static s7_pointer c_vct_offset(s7_scheme *sc, s7_pointer x, s7_double y)
-{
-  s7_int i, len;
-  s7_double *fx;
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_offset, 1, x, "a float-vector");
-  len = s7_vector_length(x);
-  if (len == 0) return(x);
-  fx = s7_float_vector_elements(x);
-  for (i = 0; i < len; i++) fx[i] += y;
-  return(x);
-}
-
-PRF_TO_PF(float_vector_offset, c_vct_offset(sc, x, y))
-
-
-static s7_pointer vct_abs_pf_a(s7_scheme *sc, s7_pointer **p)
-{								
-  s7_pf_t f;							
-  s7_pointer x;						
-  s7_int i, len;
-  s7_double *fx;
-
-  f = (s7_pf_t)(**p); (*p)++;					
-  x = f(sc, p);						
-  if (!s7_is_float_vector(x)) s7_wrong_type_arg_error(sc, S_vct_abs, 1, x, "a float-vector");
-  len = s7_vector_length(x);
-  if (len == 0) return(x);
-  fx = s7_float_vector_elements(x);
-  for (i = 0; i < len; i++) fx[i] = fabs(fx[i]);
-  return(x);
-}	
-							
-static s7_pf_t float_vector_abs_pf(s7_scheme *sc, s7_pointer expr)	
-{								
-  if ((s7_is_pair(s7_cdr(expr))) && (s7_is_null(sc, s7_cddr(expr))) && 
-      (s7_arg_to_pf(sc, s7_cadr(expr))))  
-    return(vct_abs_pf_a);			\
-  return(NULL);				
-}
-#endif
-
-
-
 #if (!HAVE_SCHEME)
   Xen_wrap_2_optional_args(g_make_vct_w, g_make_vct)
   Xen_wrap_2_args(g_vct_fill_w, g_vct_fill)
@@ -1913,15 +1765,7 @@ void mus_vct_init(void)
   Xen_define_typed_procedure(S_vct_spatter,     g_vct_spatter_w,     4, 0, 0, H_vct_spatter,           pl_rfvir);
   Xen_define_typed_procedure(S_vct_interpolate, g_vct_interpolate_w, 7, 0, 0, H_vct_interpolate,       pl_rfiir);
 
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_add), float_vector_add_pf);
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_subtract), float_vector_subtract_pf);
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_multiply), float_vector_multiply_pf);
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_scale), float_vector_scale_pf);
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_offset), float_vector_offset_pf);
-  s7_pf_set_function(s7_name_to_value(s7, S_vct_abs), float_vector_abs_pf);
-
-  s7_rf_set_function(s7_name_to_value(s7, S_vct_min), float_vector_min_rf);
-  s7_rf_set_function(s7_name_to_value(s7, S_vct_max), float_vector_max_rf);
-  s7_rf_set_function(s7_name_to_value(s7, S_vct_peak), float_vector_peak_rf);
+  s7_set_d_p_function(s7_name_to_value(s7, S_vct_min), float_vector_min_d_p);
+  s7_set_d_p_function(s7_name_to_value(s7, S_vct_max), float_vector_max_d_p);
 #endif
 }
