@@ -35,6 +35,8 @@
     (lambda (beg dur) 
       (list (seconds->samples beg) (seconds->samples (+ beg dur))))))
 
+(define *clm-default-frequency* 0.0) ; this is obsolete
+
 
 ;;; -------- definstrument --------
 
@@ -501,7 +503,7 @@
      (dynamic-wind
 	 (lambda ()
 	   (set! *clm-notehook* (lambda (name . args)
-				  (set! mark-list (cons (append (list name) args) mark-list)))))
+				  (set! mark-list (cons (cons name args) mark-list)))))
 
 	 (lambda ()
 	   (let* ((result (with-sound-helper (lambda () ,@body) ,@args))
@@ -818,7 +820,7 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
 
 (define (clm-display-globals)
 
-  (format #f ";CLM globals:~%;  *clm-srate*: ~A (default: ~A)~%;  *clm-file-name*: ~A~%;  *clm-channels: ~A (default: ~A)~%;  *clm-sample-type*: ~A (default: ~A)~%;  *clm-header-type*: ~A (default: ~A)~%;  *clm-reverb-channels*: ~A, *clm-reverb-data*: ~A~%;  *clm-table-size*: ~A~%;  *clm-file-buffer-size*: ~A~%;  *clm-locsig-type*: ~A~%;  *clm-array-print-length*: ~A (~A)~%;  *clm-notehook*: ~A~%;  *clm-default-frequency*: ~A~%;  *clm-clipped*: ~A, mus-clipping: ~A~%~%"
+  (format #f ";CLM globals:~%;  *clm-srate*: ~A (default: ~A)~%;  *clm-file-name*: ~A~%;  *clm-channels: ~A (default: ~A)~%;  *clm-sample-type*: ~A (default: ~A)~%;  *clm-header-type*: ~A (default: ~A)~%;  *clm-reverb-channels*: ~A, *clm-reverb-data*: ~A~%;  *clm-table-size*: ~A~%;  *clm-file-buffer-size*: ~A~%;  *clm-locsig-type*: ~A~%;  *clm-array-print-length*: ~A (~A)~%;  *clm-notehook*: ~A~%;  *clm-clipped*: ~A, mus-clipping: ~A~%~%"
 
 	  *clm-srate* *default-output-srate*
 	  *clm-file-name*
@@ -831,7 +833,6 @@ symbol: 'e4 for example.  If 'pythagorean', the frequency calculation uses small
 	  *clm-locsig-type*
 	  *clm-array-print-length* *print-length*
 	  *clm-notehook*
-	  *clm-default-frequency*
 	  *clm-clipped* (mus-clipping)))
 
 
